#include "MediaEncryptDecryptPlugin.h"

#include <fstream>

std::ofstream file_stream;

ITRTCPlugin* createTRTCPlugin() { return new MediaEncryptDecryptPlugin(); }

void destoryTRTCPlugin(ITRTCPlugin* plugin) {
  if (plugin) {
    delete plugin;
  }
}

MediaEncryptDecryptPlugin::MediaEncryptDecryptPlugin() {
  encrypt_key_ =
      "ahsdiFTIUDFHhKGHGVvfhgfdrFJFCJFkGHZKJgJKHFJYRFJHLJjnLKHJKHKLJ";
}

MediaEncryptDecryptPlugin::~MediaEncryptDecryptPlugin() {}

bool MediaEncryptDecryptPlugin::init() {
  std::lock_guard<std::mutex> lock(mutex_);
  if (!inited_) {
    inited_ = true;
  }
  return true;
}

bool MediaEncryptDecryptPlugin::uninit() {
  std::lock_guard<std::mutex> lock(mutex_);
  if (inited_) {
    inited_ = false;
  }
  return true;
}

bool MediaEncryptDecryptPlugin::load(const char* path) {
  std::lock_guard<std::mutex> lock(mutex_);
  if (loaded_) {
    return false;
  }
  if (path) {
    folder_path_ = path;
  }
  loaded_ = true;
  return true;
}

bool MediaEncryptDecryptPlugin::unload() {
  std::lock_guard<std::mutex> lock(mutex_);
  if (!loaded_) {
    return false;
  }

  folder_path_.clear();
  loaded_ = false;
  return true;
}

bool MediaEncryptDecryptPlugin::enable(bool enabled) {
  std::lock_guard<std::mutex> lock(mutex_);
  enabled_ = enabled;
  return true;
}

bool MediaEncryptDecryptPlugin::setParameter(const char* param) {
  std::lock_guard<std::mutex> lock(mutex_);
  if (param) {
    params_ = param;
  }
  return true;
}

TRTC_PLUGIN_TYPE MediaEncryptDecryptPlugin::getPluginType() {
  return TRTC_PLUGIN_TYPE::TRTC_PLUGIN_MEDIAENCRYPTDECRYPT;
}

void MediaEncryptDecryptPlugin::setMallocMemoryCallback(MallocMemory callback) {
  malloc_memory_callback_ = callback;
}

bool MediaEncryptDecryptPlugin::didEncodeVideo(EncodedData& data) {
  if (!enabled_) {
    return false;
  }
  return XORData(data);
}

bool MediaEncryptDecryptPlugin::willDecodeVideo(EncodedData& data) {
  if (!enabled_) {
    return false;
  }
  return XORData(data);
}

bool MediaEncryptDecryptPlugin::didEncodeAudio(EncodedData& data) {
  if (!enabled_) {
    return false;
  }
  return XORData(data);
}

bool MediaEncryptDecryptPlugin::willDecodeAudio(EncodedData& data) {
  if (!enabled_) {
    return false;
  }
  return XORData(data);
}

bool MediaEncryptDecryptPlugin::XORData(EncodedData& data) {
  uint8_t* new_data = nullptr;
  uint32_t new_size = data.originData.size;
  if (!malloc_memory_callback_) {
    return false;
  }
  bool result = malloc_memory_callback_(data.originData.data, &new_data, new_size);
  if (!result) {
    return false;
  }

  auto key_size = encrypt_key_.size();
  for (uint32_t i = 0; i < new_size; ++i) {
    new_data[i] = data.originData.data[i] ^ encrypt_key_[i % key_size];
  }

  return true;
}
